# -*- coding: utf-8 -*-
# vim: ft=python:sw=4:ts=4:sts=4:et:
import datetime
import decimal

from optparse import make_option

from django.core.management.base import BaseCommand
from django_bnr.utils import get_bnr_rate
from django_bnr.models import CURRENCIES


class Command(BaseCommand):
    help = 'Gets the BNR rate for given currency'

    def add_arguments(self, parser):
        parser.add_argument(
            "-c",
            "--currency",
            type=str,
            choices=CURRENCIES,
            dest="currency",
            action="store",
            default="USD",
            help="Currency to get rate for",
        )

        # Named (optional) arguments
        parser.add_argument(
            "-d",
            "--date",
            type=str,
            dest="date",
            action="store",
            default=datetime.date.today().strftime("%Y-%m-%d"),
            help="Date to get rate for",
        )

    def handle(self, *args, **options):
        currency = options['currency']
        date = datetime.datetime.strptime(options['date'], '%Y-%m-%d').date()
        print(get_bnr_rate(date, currency))



from django.contrib import admin
from django_bnr.models import Rate


# Register your models here.
class RateAdmin(admin.ModelAdmin):
    list_display = ('rate', 'currency', 'date')


admin.site.register(Rate, RateAdmin)



# -*- coding: utf-8 -*-
# vim: ft=python:sw=4:ts=4:sts=4:et:
import requests
import decimal

from datetime import timedelta
from xml.etree import ElementTree

from django_bnr.models import Rate


def get_bnr_rate(date, currency='USD'):
    try:
        rate = Rate.objects.get(date=date, currency=currency)
        return rate.rate
    except Rate.DoesNotExist:
        d = date.strftime('%Y-%m-%d')
        r = requests.get('https://www.bnr.ro/nbrfxrates10days.xml')
        r.raise_for_status()
        rate = None
        days = 0
        xpath_fmt = ("./{xsd}Body/{xsd}Cube[@date='{date}']/"
                     "{xsd}Rate[@currency='{currency}']")
        while rate is None:
            rate = ElementTree.fromstring(r.text).find(xpath_fmt.format(
                xsd='{http://www.bnr.ro/xsd}',
                date=d,
                currency=currency
            ))
            if rate is None:
                days += 1
                if days == 7:
                    raise RuntimeError('Cannot get exchange rate for '
                                       '%(currency)s from %(date)s' % {
                                           'currency': currency,
                                           'date': date
                                       })
                d = (date - timedelta(days=days)).strftime('%Y-%m-%d')

        rate = decimal.Decimal(rate.text)
        try:
            Rate.objects.create(date=date, currency=currency, rate=rate)
        except:
            pass
        return rate



from django.db import models

CURRENCIES = ('USD', 'EUR', 'CHF', 'GBP')


class Rate(models.Model):
    rate = models.DecimalField(max_digits=8, decimal_places=4, null=True,
                               blank=True, verbose_name=('Exchange rate'))
    date = models.DateField(db_index=True)
    currency = models.CharField(choices=((c, c) for c in sorted(CURRENCIES)),
                                max_length=3, db_index=True, default='USD')

    class Meta:
        ordering = ['-date', 'currency']
        unique_together = ('date', 'currency')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Rate',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('rate', models.DecimalField(null=True, verbose_name=b'Exchange rate', max_digits=8, decimal_places=4, blank=True)),
                ('date', models.DateField(db_index=True)),
                ('currency', models.CharField(default=b'USD', max_length=3, db_index=True, choices=[(b'CHF', b'CHF'), (b'EUR', b'EUR'), (b'GBP', b'GBP'), (b'USD', b'USD')])),
            ],
            options={
                'ordering': ['-date', 'currency'],
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='rate',
            unique_together=set([('date', 'currency')]),
        ),
    ]
